( cefSIM328.F, 15jan11cht, for Arduino Uno )

( Map RAM to 0-8FF in cCODE array )
( RESET gets starting address from $900 in cCODE array, 29oct11cht )
( All numbers are 32-bit, no need for carry extension)

HEX

$1F CONSTANT LIMIT ( stack depth )
$7FFF CONSTANT RANGE ( program memory size in words )
VARIABLE CLOCK ( slot is in the last 2 bits )
VARIABLE BREAK

CREATE REGISTER $300 ALLOT
: C+! DUP >R C@ + R> C! ;

DECIMAL
REGISTER CONSTANT P
REGISTER 4 + CONSTANT I
REGISTER 8 + CONSTANT I1
REGISTER 9 + CONSTANT I2
REGISTER 13 + CONSTANT RP
REGISTER 14 + CONSTANT SP
REGISTER 16 + CONSTANT T
REGISTER 24 + CONSTANT IP
REGISTER $100 + CONSTANT RSTACK0
REGISTER $200 + CONSTANT SSTACK0

HEX
: RSTACK  RP C@ LIMIT AND 2 LSHIFT RSTACK0 + ;
: SSTACK  SP C@ LIMIT AND 2 LSHIFT SSTACK0 + ;

: CYCLE 1 CLOCK +! ;
: JUMP  CLOCK @ 3 OR CLOCK ! ;
: RPUSH ( n -- , push n on return stack )
	1 RP C+! RSTACK W! ;
: RPOPP ( -- n , pop n from return stack )
	RSTACK W@ -1 RP C+! ;
: SPUSH ( n -- , push n on data stack )
	1 SP C+! 
	T W@ SSTACK W!
	T ! ;
: SPOPP ( -- n , pop n from data stack )
	T W@
	SSTACK W@ T W! 
	-1 SP C+! ;
: continue
	P W@ RAM@ DUP I W!
	100 /MOD SWAP I1 C!
	FF AND I2 C!
	2 P +! ;

: next,	IP W@ RAM@ P W!
	2 IP +! JUMP ;
: nop,    JUMP ;
: bye,    ABORT" Simulation done." ;
: qrx,    KEY SPUSH -1 SPUSH ;
: txsto,  SPOPP TX! ;
: inline, P W@ RAM@ SPUSH 2 P +! ;
: dolit,  IP W@ RAM@ SPUSH 2 IP +! next, ;
: dolist, IP W@ RPUSH P W@ IP W! next, ;
: exit,   RPOPP IP W! next, ;
: execu,  IP W@ RPUSH SPOPP P W! JUMP ;
: donext, RPOPP ?DUP IF 1- RPUSH IP W@ RAM@ IP W!
	  ELSE 2 IP +! THEN next, ;
: qbran,  SPOPP IF 2 IP +! ELSE IP W@ RAM@ IP ! THEN next, ;
: bran,   IP W@ RAM@ IP W! next, ;
: store,  SPOPP SPOPP SWAP RAM! ;
: at,     SPOPP RAM@ SPUSH ;
: istore,  SPOPP SPOPP SWAP RAM! ;
: iat,     SPOPP RAM@ SPUSH ;
: icat,     SPOPP RAMC@ SPUSH ;
: cstor,  SPOPP SPOPP SWAP RAMC! ;
: cat,    SPOPP RAMC@ SPUSH ;
: rfrom,  RPOPP SPUSH ;
: rat,    RPOPP DUP RPUSH SPUSH ;
: tor,    SPOPP RPUSH ;
: drop,   SPOPP DROP ;
: dup,    SPOPP DUP SPUSH SPUSH ;
: swap,   SPOPP SPOPP SWAP SPUSH SPUSH ;
: over,   SPOPP SPOPP DUP SPUSH SWAP SPUSH SPUSH ;
: zless,  SPOPP $8000 AND $8000 = $FFFF AND SPUSH ;
: andd,   SPOPP SPOPP AND SPUSH ;
: orr,    SPOPP SPOPP OR SPUSH ;
: xorr,   SPOPP SPOPP XOR SPUSH ;
: uplus,  SPOPP SPOPP + DUP $FFFF AND SPUSH
          $10000 AND IF 1 ELSE 0 THEN SPUSH ;
: dovar,  P @ SPUSH ;

HEX

: ERR  . 1 ABORT" :Illegel :ruction" ;

CREATE CODE-TABLE
' nop,    , ' bye,    , ' qrx,    , ' txsto, , 
' inline, , ' dolit,  , ' dolist, , ' exit,  , 
' execu,  , ' donext, , ' qbran,  , ' bran,  , 
' store,  , ' at,     , ' cstor,  , ' cat,   , 
' istore, , ' iat,    , ' rfrom,  , ' rat,   , 
' tor,    , ' dovar,  , ' next,   , ' drop,  , 
' dup,    , ' swap,   , ' over,   , ' zless, , 
' andd,   , ' orr,    , ' xorr,   , ' uplus, , 
' icat,   ,

: executecode ( code -- )
   DUP 21 > ABORT" Illegal code "
   CELLS CODE-TABLE + @ EXECUTE ;

: .stack ( add # ) FOR AFT DUP @ U. 4 - THEN NEXT DROP CR ;
: .sstack ." S:" T @ U.
        SSTACK SP C@ .stack ;
: .rstack ." R:" RSTACK RP C@ .stack ;
: .registers ."  IP=" IP @ . ."  P=" P @ . ."  I=" I @ U.
        ."  I1=" I1 C@ . ."  I2=" I2 C@ .
        CR ;
: S  ."  CLOCK=" CLOCK @ . .registers
        .sstack .rstack ;

: SYNC0  continue ;
: SYNC1  I1 C@ executecode ;
: SYNC2  I2 C@ executecode ;

CREATE SYNC-TABLE
' continue , ' SYNC1 , ' SYNC2 , ' JUMP ,

: sync  CLOCK @ 3 AND cells
        SYNC-TABLE + @ EXECUTE ;
: C     sync CYCLE S ;
: RESET REGISTER $300 ERASE 0 CLOCK ! 
	$900 RAM@ P ! ( start of code table in flash )
	;
RESET

: G     ( addr -- )
        CR ." Press any key to stop." CR
        BREAK !
        BEGIN sync P @ BREAK @ =
              IF CYCLE C EXIT
              ELSE CYCLE
              THEN
              ?KEY
        UNTIL ;
: PUSH  ( n ) T @ SPUSH T ! ;
: POP   SPOPP ;

: D     P @ CELL- FOUR FOUR ;
: M     SHOW ;
: RUN   CR ." Press ESC to stop." CR
        BEGIN C KEY 1B = UNTIL ;

: HELP  CR ." cEF Simulator, copyright Offete Enterprises, 2009"
        CR ." C: execute next cycle"
        CR ." S: show all registers"
        CR ." D: display next 8 words"
        CR ." addr M: display 128 words from addr"
        CR ." addr G: run and stop at addr"
        CR ." RUN: execute, one key per cycle"
        CR ;

